//+------------------------------------------------------------------+
//|                                                IncATROnArray.mqh |
//|                                                          Integer |
//|                          https://login.mql5.com/en/users/Integer |
//+------------------------------------------------------------------+
#property copyright "Integer"
#property link      "https://login.mql5.com/en/users/Integer"
#property version   "1.00"
/*
   External parameters:
   input int            ATRPeriod      =  14;
   input ENUM_MA_METHOD ATRMethod      =  MODE_SMA;

   Declaration: 
   #include <IncOnArray/IncATROnArray.mqh>
   CATROnArray atr;
   
   In OnInit:   
   atr.Init(ATRPeriod,ATRMethod);
   
   In OnCalculate:
   atr.Solve(rates_total,prev_calculated,DataHigh,DataLow,DataClose,TRBuffer,ATRBuffer);
*/

#include <IncOnArray/IncMAOnArray.mqh>
//+------------------------------------------------------------------+
//| CATROnArray                                                      |
//+------------------------------------------------------------------+
class CATROnArray
  {
private:
   int               m_Period;
   ENUM_MA_METHOD    m_Method;
   string            m_Name;
   CMAOnArray        m_ma;
   int               m_br;
public:
   void Init(int aPeriod=14,ENUM_MA_METHOD aMethod=MODE_SMA)
     {
      m_Period=aPeriod;
      m_Method=aMethod;
      m_ma.Init(m_Period,m_Method);
      m_Name="ATR("+IntegerToString(aPeriod)+","+m_ma.NameMethod()+")";
      m_br=m_ma.BarsRequired()+1;
     }
   void Solve(const int aRatesTotal,
              const int aPrevCalc,
              double  &aDataHigh[],
              double  &aDataLow[],
              double  &aDataClose[],
              double  &aTR[],
              double  &aATR[])
     {
      int Start=0;
      if(aPrevCalc==0)
        {
         for(int i=0;i<aRatesTotal;i++)
           {
            if(aDataHigh[i]!=0 && aDataHigh[i]!=EMPTY_VALUE && 
               aDataLow[i]!=0 && aDataLow[i]!=EMPTY_VALUE && 
               aDataClose[i]!=0 && aDataClose[i]!=EMPTY_VALUE)
              {
               Start=i+1;
               break;
              }
           }
        }
      else
        {
         Start=aPrevCalc-1;
        }
      for(int i=Start;i<aRatesTotal;i++)
        {
         aTR[i]=MathMax(aDataHigh[i],aDataClose[i-1])-MathMin(aDataLow[i],aDataClose[i-1]);
        }
      m_ma.Solve(aRatesTotal,aPrevCalc,aTR,aATR);
     }
   int BarsRequired()
     {
      return(m_br);
     }
   string Name()
     {
      return(m_Name);
     }
   string About()
     {
      return("Integer's ATROnArray class. https://login.mql5.com/en/users/Integer");
     }
  };
//+------------------------------------------------------------------+
